@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2013-2022 Richard Hull and contributors
+# Copyright (c) 2013-2023 Richard Hull and contributors
 # See LICENSE.rst for details.
 
 """
@@ -47,7 +47,10 @@ from luma.lcd.segment_mapper import dot_muncher
 from luma.core.virtual import character
 from luma.core.bitmap_font import embedded_fonts
 
-__all__ = ["pcd8544", "st7735", "st7789", "ht1621", "uc1701x", "st7567", "ili9341", "ili9486", "hd44780"]
+__all__ = [
+    "pcd8544", "st7735", "st7789", "ht1621", "uc1701x", "st7567", "ili9341",
+    "ili9486", "ili9488", "hd44780"
+]
 
 
 class GPIOBacklight:
@@ -57,14 +60,14 @@ class GPIOBacklight:
     :param gpio: GPIO interface (must be compatible with `RPi.GPIO <https://pypi.python.org/pypi/RPi.GPIO>`_).
     :param pin: the GPIO pin to use for the backlight.
     :type pin: int
-    :param active_low: Set to True if active low (default), False otherwise.
+    :param active_low: Set to True if active low, False otherwise (default).
     :type active_low: bool
     :raises luma.core.error.UnsupportedPlatform: GPIO access not available.
 
     .. versionadded:: 2.3.0
     """
 
-    def __init__(self, gpio, pin=18, active_low=True):
+    def __init__(self, gpio, pin=18, active_low=False):
         self._gpio = gpio
         self._pin = pin
         if active_low:
@@ -187,12 +190,12 @@ class PWMBacklight:
 @rpi_gpio
 class backlit_device(device):
     """
-    Controls a backlight (active low), assumed to be on GPIO 18 (``PWM_CLK0``) by default.
+    Controls a backlight (active high), assumed to be on GPIO 18 (``PWM_CLK0``) by default.
 
     :param gpio: GPIO interface (must be compatible with `RPi.GPIO <https://pypi.python.org/pypi/RPi.GPIO>`_).
     :param gpio_LIGHT: The GPIO pin to use for the backlight.
     :type gpio_LIGHT: int
-    :param active_low: Set to true if active low (default), false otherwise.
+    :param active_low: Set to true if active low, false otherwise (default).
     :type active_low: bool
     :param pwm_frequency: Use PWM for backlight brightness control with the specified frequency when provided.
     :type pwm_frequency: float
@@ -204,10 +207,12 @@ class backlit_device(device):
     .. versionadded:: 2.0.0
     """
 
-    def __init__(self, const=None, serial_interface=None, gpio=None, gpio_LIGHT=18, active_low=True, pwm_frequency=None, backpack_pin=None, **kwargs):
-        super(backlit_device, self).__init__(const, serial_interface)
+    def __init__(self, const=None, serial_interface=None, gpio=None, gpio_LIGHT=18, active_low=False, pwm_frequency=None, backpack_pin=None, backlight=None, **kwargs):
+        super(backlit_device, self).__init__(const, serial_interface, **kwargs)
 
-        if backpack_pin or (isinstance(serial_interface, pcf8574) and hasattr(serial_interface, "_backlight_enabled")):
+        if backlight:
+            self.backlight = backlight
+        elif backpack_pin or (isinstance(serial_interface, pcf8574) and hasattr(serial_interface, "_backlight_enabled")):
             self.backlight = I2CBackpackBacklight(serial_interface, pin=backpack_pin)
         elif pwm_frequency:
             self._gpio = gpio or self.__rpi_gpio__()
@@ -216,7 +221,6 @@ class backlit_device(device):
             self._gpio = gpio or self.__rpi_gpio__()
             self.backlight = GPIOBacklight(self._gpio, pin=gpio_LIGHT, active_low=active_low)
 
-        self.persist = True
         self.backlight(True)
 
     def cleanup(self):
@@ -224,7 +228,7 @@ class backlit_device(device):
         Attempt to reset the device & switching it off prior to exiting the
         python process.
         """
-        if self.persist:
+        if not self.persist:
             self.backlight(False)
         try:
             self.backlight.cleanup()
@@ -513,10 +517,14 @@ class st7735(backlit_device, __framebuffer_mixin):
             self.apply_offsets = lambda bbox: bbox
 
         # Supported modes
-        supported = (width, height) in [(160, 80), (160, 128), (128, 128)]
+        supported = (width, height) in [
+            (160, 80),
+            (160, 128),
+            (128, 128)
+        ]
         if not supported:
             raise luma.core.error.DeviceDisplayModeError(
-                "Unsupported display mode: {0} x {1}".format(width, height))
+                f"Unsupported display mode: {width} x {height}")
 
         # RGB or BGR order
         order = 0x08 if bgr else 0x00
@@ -640,10 +648,14 @@ class ili9341(backlit_device, __framebuffer_mixin):
             self.apply_offsets = lambda bbox: bbox
 
         # Supported modes
-        supported = (width, height) in [(320, 240), (240, 240), (320, 180)]  # full, 1x1, 16x9
+        supported = (width, height) in [
+            (320, 240),  # full
+            (240, 240),  # 1x1
+            (320, 180)   # 16x9
+        ]
         if not supported:
             raise luma.core.error.DeviceDisplayModeError(
-                "Unsupported display mode: {0} x {1}".format(width, height))
+                f"Unsupported display mode: {width} x {height}")
 
         # RGB or BGR order
         order = 0x00 if bgr else 0x08
@@ -751,6 +763,8 @@ class ili9486(backlit_device, __framebuffer_mixin):
     :type framebuffer: luma.core.framebuffer.framebuffer
     :param bgr: Set to ``True`` if device pixels are BGR order (rather than RGB).
     :type bgr: bool
+    :param invert: Set to ``False`` if device does not require inverted colors.
+    :type invert: bool
     :param h_offset: Horizontal offset (in pixels) of screen to device memory
         (default: 0).
     :type h_offset: int
@@ -762,7 +776,8 @@ class ili9486(backlit_device, __framebuffer_mixin):
     """
 
     def __init__(self, serial_interface=None, width=320, height=480, rotate=0,
-                 framebuffer=None, h_offset=0, v_offset=0, bgr=False, **kwargs):
+                 framebuffer=None, h_offset=0, v_offset=0, bgr=False, invert=True,
+                 **kwargs):
         super(ili9486, self).__init__(luma.lcd.const.ili9486, serial_interface, **kwargs)
         self.capabilities(width, height, rotate, mode="RGB")
         self.init_framebuffer(framebuffer, 25)
@@ -776,10 +791,12 @@ class ili9486(backlit_device, __framebuffer_mixin):
             self.apply_offsets = lambda bbox: bbox
 
         # Supported modes
-        supported = (width, height) in [(320, 480)]  # full
+        supported = (width, height) in [
+            (320, 480)  # full
+        ]
         if not supported:
             raise luma.core.error.DeviceDisplayModeError(
-                "Unsupported display mode: {0} x {1}".format(width, height))
+                f"Unsupported display mode: {width} x {height}")
 
         # RGB or BGR order
         order = 0x00 if bgr else 0x08
@@ -810,7 +827,10 @@ class ili9486(backlit_device, __framebuffer_mixin):
         self.command(0x11)                                  # sleep out
         sleep(0.150)
         self.command(0x3a, 0x00, 0x66)                      # Interface Pixel Format 6-6-6
-        self.command(0x21)                                  # Display inversion ON for LCD(B)
+
+        if invert:
+            self.command(0x21)                              # Display inversion ON for LCD(B)
+
         self.command(0xc0, 0x00, 0x09, 0x00, 0x09)          # Power Control 1
         self.command(0xc1, 0x00, 0x41, 0x00, 0x00)          # Power Control 2
         self.command(0xc2, 0x00, 0x33)                      # Power Control 3 (for normal mode)
@@ -879,6 +899,131 @@ class ili9486(backlit_device, __framebuffer_mixin):
             self._serial_interface.data(list(args))
 
 
+class ili9488(backlit_device, __framebuffer_mixin):
+    """
+    Serial interface to a 262k color (6-6-6 RGB) ILI9488 LCD display.
+
+    On creation, an initialization sequence is pumped to the display to properly
+    configure it. Further control commands can then be called to affect the
+    brightness and other settings.
+
+    :param serial_interface: the serial interface (usually a
+        :py:class:`luma.core.interface.serial.spi` instance) to delegate sending
+        data and commands through.
+    :param width: The number of pixels laid out horizontally.
+    :type width: int
+    :param height: The number of pixels laid out vertically.
+    :type height: int
+    :param rotate: An integer value of 0 (default), 1, 2 or 3 only, where 0 is
+        no rotation, 1 is rotate 90° clockwise, 2 is 180° rotation and 3
+        represents 270° rotation.
+    :type rotate: int
+    :param framebuffer: Framebuffering strategy, currently instances of
+        ``diff_to_previous()`` or ``full_frame()`` are only supported.
+    :type framebuffer: luma.core.framebuffer.framebuffer
+    :param bgr: Set to ``True`` if device pixels are BGR order (rather than RGB).
+    :type bgr: bool
+    :param h_offset: Horizontal offset (in pixels) of screen to device memory
+        (default: 0).
+    :type h_offset: int
+    :param v_offset: Vertical offset (in pixels) of screen to device memory
+        (default: 0).
+    :type v_offset: int
+
+    .. versionadded:: 2.11.0
+    """
+
+    def __init__(self, serial_interface=None, width=480, height=320, rotate=0,
+                 framebuffer=None, h_offset=0, v_offset=0, bgr=False, **kwargs):
+        super(ili9488, self).__init__(luma.lcd.const.ili9488, serial_interface, **kwargs)
+        self.capabilities(width, height, rotate, mode="RGB")
+        self.init_framebuffer(framebuffer, 25)
+
+        if h_offset != 0 or v_offset != 0:
+            def offset(bbox):
+                left, top, right, bottom = bbox
+                return (left + h_offset, top + v_offset, right + h_offset, bottom + v_offset)
+            self.apply_offsets = offset
+        else:
+            self.apply_offsets = lambda bbox: bbox
+
+        # Supported modes
+        supported = (width, height) in [(480, 320)]
+        if not supported:
+            raise luma.core.error.DeviceDisplayModeError(
+                f"Unsupported display mode: {width} x {height}")
+
+        # RGB or BGR order
+        order = 0x00 if bgr else 0x08
+
+        # Initialization sequence, adapted from
+        #
+        # https://github.com/birdtechstep/fbtft/blob/master/fb_ili9488.c
+        #
+
+        self.command(0xe0, 0x00, 0x03, 0x09, 0x08, 0x16, 0x0a, 0x37,   # Positive Gamma Control
+                     0x78, 0x4c, 0x09, 0x0a, 0x08, 0x16, 0x1a, 0x0f)
+        self.command(0xe1, 0x00, 0x16, 0x19, 0x03, 0x0f, 0x05, 0x32,
+                     0x45, 0x46, 0x04, 0x0e, 0x0d, 0x35, 0x37, 0x0f)  # Negative Gamma Control
+        self.command(0xc0, 0x17, 0x15)                                 # Power Control 1
+        self.command(0xc1, 0x41)                                       # Power Control 2
+        self.command(0xc5, 0x00, 0x12, 0x80)                           # VCOM Control
+        self.command(0x36, 0x20 | order)                               # Memory Access Control
+        self.command(0x3a, 0x66)                                       # Interface Pixel Format 6-6-6
+        self.command(0xb0, 0x00)                                       # Interface Mode Control
+        self.command(0xb1, 0xa0)                                       # Frame Rate Control
+        self.command(0xb4, 0x02)                                       # Display Inversion Control
+        self.command(0xb6, 0x02, 0x02, 0x3b)                           # Display Function Control
+        self.command(0xb7, 0xc6)                                       # Entry Mode Set
+        self.command(0xf7, 0xa9, 0x51, 0x2c, 0x82)                     # Interface Mode Control
+        self.command(0x11)                                             # Sleep Out
+        sleep(0.120)
+        self.clear()
+        self.show()
+
+    def display(self, image):
+        """
+        Renders a 24-bit RGB image to the ILI9488 LCD display. The 8-bit RGB
+        values are passed directly to the devices internal storage, but only
+        the 6 most-significant bits are used by the display.
+
+        :param image: The image to render.
+        :type image: PIL.Image.Image
+        """
+        assert image.mode == self.mode
+        assert image.size == self.size
+
+        image = self.preprocess(image)
+
+        for image, bounding_box in self.framebuffer.redraw(image):
+            left, top, right, bottom = self.apply_offsets(bounding_box)
+
+            self.command(0x2a, left >> 8, left & 0xff, (right - 1) >> 8, (right - 1) & 0xff)     # Set column addr
+            self.command(0x2b, top >> 8, top & 0xff, (bottom - 1) >> 8, (bottom - 1) & 0xff)     # Set row addr
+            self.command(0x2c)                                                                   # Memory write
+
+            self.data(image.tobytes())
+
+    def contrast(self, level):
+        """
+        NOT SUPPORTED
+
+        :param level: Desired contrast level in the range of 0-255.
+        :type level: int
+        """
+        assert 0 <= level <= 255
+
+    def command(self, cmd, *args):
+        """
+        Sends a command and an (optional) sequence of arguments through to the
+        delegated serial interface. Note that the arguments are passed through
+        as data.
+        """
+        self._serial_interface.command(cmd)
+        if len(args) > 0:
+            self._serial_interface.data(list(args))
+
+
 @rpi_gpio
 class ht1621(backlit_device):
     """
@@ -1102,8 +1247,8 @@ class hd44780(backlit_device, parallel_device, character, __framebuffer_mixin):
     :param gpio_LIGHT: The GPIO pin to use for the backlight if it is controlled by
         one of the GPIO pins.
     :type gpio_LIGHT: int
-    :param active_low: Set to true if backlight is active low (default), false
-        otherwise.
+    :param active_low: Set to true if backlight is active low, false
+        otherwise (default).
     :type active_low: bool
     :param pwm_frequency: Use PWM for backlight brightness control with the
         specified frequency when provided.
@@ -1115,6 +1260,9 @@ class hd44780(backlit_device, parallel_device, character, __framebuffer_mixin):
     :param framebuffer: Framebuffering strategy, currently instances of
         ``diff_to_previous()`` or ``full_frame()`` are only supported.
     :type framebuffer: luma.core.framebuffer.framebuffer
+    :param backlight: The serial interface (usually a
+        :py:class:`luma.core.interface.serial.parallel` instance) to delegate
+        sending backlight control commands through.
 
     To place text on the display, simply assign the text to the ``text``
     instance variable::
@@ -1133,8 +1281,8 @@ class hd44780(backlit_device, parallel_device, character, __framebuffer_mixin):
     """
 
     def __init__(self, serial_interface=None, width=16, height=2, undefined='_',
-                 selected_font=0, exec_time=0.000001, framebuffer=None, **kwargs):
-        super(hd44780, self).__init__(luma.lcd.const.hd44780, serial_interface,
+                 selected_font=0, exec_time=0.000001, framebuffer=None, backlight=None, **kwargs):
+        super(hd44780, self).__init__(luma.lcd.const.hd44780, serial_interface, backlight=backlight,
         exec_time=exec_time, **kwargs)
 
         # Inherited from parallel_device class but multi-inheritence with
@@ -1153,10 +1301,15 @@ class hd44780(backlit_device, parallel_device, character, __framebuffer_mixin):
         self._custom = {}
 
         # Supported modes
-        supported = (width, height) in [(16, 2), (16, 4), (20, 2), (20, 4)]
+        supported = (width, height) in [
+            (16, 2),
+            (16, 4),
+            (20, 2),
+            (20, 4)
+        ]
         if not supported:
             raise luma.core.error.DeviceDisplayModeError(
-                "Unsupported display mode: {0} x {1}".format(width, height))
+                f"Unsupported display mode: {width} x {height}")
 
         self._initialize_device()
         self.text = ''
@@ -1224,6 +1377,7 @@ class hd44780(backlit_device, parallel_device, character, __framebuffer_mixin):
         dl = self._const.DL8 if self._bitmode == 8 else self._const.DL4
         self.command(self._const.FUNCTIONSET | dl | self._const.LINES2)
         self.command(self._const.DISPLAYOFF)  # Set Display Off
+        self.command(self._const.CLEAR, exec_time=1e-3 * 1.6)  # Clear display
         self.command(self._const.ENTRY)  # Set entry mode to right, no shift
         self.command(self._const.DISPLAYON, exec_time=1e-3 * 100)  # Turn display on
 
